﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Core;
using HeuristicLab.Collections;

namespace HeuristicLab.Problems.MetaOptimization {
  public class NumericParameterConfiguration : ParameterConfiguration {
    private NumericRangeList ranges;
    public NumericRangeList Ranges {
      get { return ranges; }
    }

    protected NumericParameterConfiguration() { }
    public NumericParameterConfiguration(IParameter parameter, string category)
      : base(parameter, category) {
      this.ranges = new NumericRangeList();
      this.ranges.ItemsAdded += new CollectionItemsChangedEventHandler<INumericRange>(ranges_ItemsAdded);
      this.ranges.ItemsRemoved += new CollectionItemsChangedEventHandler<INumericRange>(ranges_ItemsRemoved);
      this.ranges.CollectionReset += new CollectionItemsChangedEventHandler<INumericRange>(ranges_CollectionReset);
    }

    void ranges_CollectionReset(object sender, CollectionItemsChangedEventArgs<INumericRange> e) {
      foreach (var i in e.Items) {
        DeregisterItemEvents(i);
      }
      OnToStringChanged();
    }

    void ranges_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<INumericRange> e) {
      foreach (var i in e.Items) {
        DeregisterItemEvents(i);
      }
      OnToStringChanged();
    }

    void ranges_ItemsAdded(object sender, CollectionItemsChangedEventArgs<INumericRange> e) {
      foreach (var i in e.Items) {
        RegisterItemEvents(i);
      }
      OnToStringChanged();
    }

    private void DeregisterItemEvents(INumericRange range) {
      range.ToStringChanged -= new EventHandler(range_ToStringChanged);
    }
    private void RegisterItemEvents(INumericRange range) {
      range.ToStringChanged += new EventHandler(range_ToStringChanged);
    }

    public IntValue GetRandomValue() {
      return new IntValue(0);
    }

    public override Common.IDeepCloneable Clone(Common.Cloner cloner) {
      NumericParameterConfiguration clone = (NumericParameterConfiguration)base.Clone(cloner);
      clone.ranges = new NumericRangeList();
      foreach (INumericRange range in this.ranges) {
        clone.ranges.Add(range);
      }
      return clone;
    }

    void range_ToStringChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }

    public override string ToString() {
      return string.Format("{0}: ({1})", base.ToString(), string.Join(",", ranges.Select(r => r.ToString()).ToArray()));
    }

    public override IParameter GetParameterWithRandomValue(IRandom random) {
      int rangeIndex = random.Next(ranges.Count);
      int value = random.Next(ranges.ElementAt(rangeIndex).LowerBound.Value, ranges.ElementAt(rangeIndex).UpperBound.Value); // TODO: respect StepSize parameter
      IParameter clone = (IParameter)this.Parameter.Clone();
      clone.ActualValue = new IntValue(value);
      return clone;
    }
  }
}
